==> robot_description_builder-0.0.1.tar.gz: robot_description_builder-0.0.1/local_dependencies/robot-description-builder/src/link/geometry.rs <==
mod box_geometry;
mod cylinder_geometry;
mod geometry_shape_data;
mod mesh_geometry;
use nalgebra::Matrix3;
mod sphere_geometry;

pub use box_geometry::BoxGeometry;
pub use cylinder_geometry::CylinderGeometry;
pub use mesh_geometry::MeshGeometry;
pub use sphere_geometry::SphereGeometry;

pub use geometry_shape_data::GeometryShapeData;

use std::fmt::Debug;

use crate::transform::Mirror;

use self::geometry_shape_data::GeometryShapeContainer;

pub trait BoxedMirror {
	fn boxed_mirrored(
		&self,
		mirror_matrix: &Matrix3<f32>,
	) -> Box<dyn GeometryInterface + Sync + Send>;
}

impl<Geometry> BoxedMirror for Geometry
where
	Geometry: GeometryInterface + Mirror + Sync + Send,
{
	fn boxed_mirrored(
		&self,
		mirror_matrix: &Matrix3<f32>,
	) -> Box<dyn GeometryInterface + Sync + Send> {
		self.mirrored(mirror_matrix).boxed_clone()
	}
}

/// An interface for working with `Geometry`s generically.
///
/// LONGTERM-TODO: DECIDE IF `Box<dyn dyn GeometryInterface + Sync + Send>` shoudl be replaced with [`GeometryShapeContainer`]
pub trait GeometryInterface: Debug + BoxedMirror {
	fn volume(&self) -> f32;
	fn surface_area(&self) -> f32;
	fn boxed_clone(&self) -> Box<dyn GeometryInterface + Sync + Send>;

	/// Get's the untransformed boundingbox size of the geometry from it's center. (X, Y, Z)
	fn bounding_box(&self) -> (f32, f32, f32);

	/// Gets a `GeometryShapeContainer` of the current Shape
	fn shape_container(&self) -> GeometryShapeContainer;
}

impl Mirror for Box<dyn GeometryInterface + Sync + Send> {
	fn mirrored(&self, mirror_matrix: &Matrix3<f32>) -> Self {
		self.boxed_mirrored(mirror_matrix)
	}
}

impl PartialEq for (dyn GeometryInterface + Sync + Send) {
	fn eq(&self, other: &Self) -> bool {
		// Should probably just get shape data
		self.volume() == other.volume()
			&& self.surface_area() == other.surface_area()
			&& self.bounding_box() == other.bounding_box()
	}
}

// TODO: Is this ever used?
impl From<&(dyn GeometryInterface + Sync + Send)> for Box<dyn GeometryInterface + Sync + Send> {
	fn from(value: &(dyn GeometryInterface + Sync + Send)) -> Self {
		value.boxed_clone()
	}
}

==> robot_description_builder-0.0.2.tar.gz: robot_description_builder-0.0.2/local_dependencies/robot-description-builder/src/link/geometry.rs <==
mod box_geometry;
mod cylinder_geometry;
mod geometry_shape_data;
mod mesh_geometry;
use nalgebra::Matrix3;
mod sphere_geometry;

pub use box_geometry::BoxGeometry;
pub use cylinder_geometry::CylinderGeometry;
pub use mesh_geometry::MeshGeometry;
pub use sphere_geometry::SphereGeometry;

pub use geometry_shape_data::GeometryShapeData;

// TODO: Maybe only exported for `wrapping` feature
pub use geometry_shape_data::GeometryShapeContainer;

use std::fmt::Debug;

use crate::transform::Mirror;

// use self::geometry_shape_data::GeometryShapeContainer;

pub trait BoxedMirror {
	fn boxed_mirrored(
		&self,
		mirror_matrix: &Matrix3<f32>,
	) -> Box<dyn GeometryInterface + Sync + Send>;
}

impl<Geometry> BoxedMirror for Geometry
where
	Geometry: GeometryInterface + Mirror + Sync + Send,
{
	fn boxed_mirrored(
		&self,
		mirror_matrix: &Matrix3<f32>,
	) -> Box<dyn GeometryInterface + Sync + Send> {
		self.mirrored(mirror_matrix).boxed_clone()
	}
}

/// An interface for working with `Geometry`s generically.
///
/// LONGTERM-TODO: DECIDE IF `Box<dyn dyn GeometryInterface + Sync + Send>` shoudl be replaced with [`GeometryShapeContainer`]
pub trait GeometryInterface: Debug + BoxedMirror {
	fn volume(&self) -> f32;
	fn surface_area(&self) -> f32;
	fn boxed_clone(&self) -> Box<dyn GeometryInterface + Sync + Send>;

	/// Get's the untransformed boundingbox size of the geometry from it's center. (X, Y, Z)
	fn bounding_box(&self) -> (f32, f32, f32);

	/// Gets a `GeometryShapeContainer` of the current Shape
	fn shape_container(&self) -> GeometryShapeContainer;
}

impl Mirror for Box<dyn GeometryInterface + Sync + Send> {
	fn mirrored(&self, mirror_matrix: &Matrix3<f32>) -> Self {
		self.boxed_mirrored(mirror_matrix)
	}
}

impl PartialEq for (dyn GeometryInterface + Sync + Send) {
	fn eq(&self, other: &Self) -> bool {
		// Should probably just get shape data
		self.volume() == other.volume()
			&& self.surface_area() == other.surface_area()
			&& self.bounding_box() == other.bounding_box()
	}
}

impl From<&(dyn GeometryInterface + Sync + Send)> for Box<dyn GeometryInterface + Sync + Send> {
	fn from(value: &(dyn GeometryInterface + Sync + Send)) -> Self {
		value.boxed_clone()
	}
}
